package pep;

import java.io.*; import java.util.*;
import static pep.Main.nBlocks;  import static pep.Main.isReplace;
import static pep.Main.nSamples; import static pep.Main.isPaired;

/**This class creates the randomized trial order for each participant.*/
public class DataSet{
 /**number of task phases*/   private int nPhases = nBlocks.length;
 /**file reader*/             private BufferedReader br;
 /**trial counter*/           private int nTrials = 0;
 /**random number generator*/ private Random r = new Random();
 /**final array of trials*/   private String set[] = new String[1002];
 /**loop counter*/            private int counter;
 /**task phase counter*/      private int phase = 0;
 /**array of trials*/
 private String trials[][] = new String[nPhases][1002];
 /**second trial array*/
 private String trialsB[][] = new String[nPhases][1002];

 /**This method uses a source file to read what trials occur in a block,
  * randomizes them, and saves the complete sequence of trials to an array.
  * @param source The name of the text file containing the task stimuli.
  * @param subj The participant number.*/
 DataSet(String source,int subj){
  try{
   //setup input reader
   br = new BufferedReader(new FileReader(source));
   counter = 0;

   //read each line in the file and save to the trials[] array
   for(boolean isFileEnd=false; !isFileEnd; nTrials++){
    trials[phase][nTrials] = br.readLine();
    //recording stops at "END_OF_FILE" marker
    switch(trials[phase][nTrials]){
     case "END_OF_FILE":
      //stop the looping
      isFileEnd=true;
      if(Main.twoTasks) splitTrials();
      //randomize the trials
      mixTrials();
      break;
     case "END_OF_BLOCK":
      if(Main.twoTasks) splitTrials();
      //randomize the trial set
      mixTrials();
      nTrials=-1;
      //increment task phase counter
      phase++;
      break;
    }
   }
   //leave an "END_OF_FILE" marker and close processes
   set[counter] = "END_OF_FILE";
   br.close();
  }
  catch(Exception e){
   System.out.println("Error reading source");
  }
 }

 /**This method randomizes the set of trials sent to it with one of several
  * randomization routines.*/
 private void mixTrials(){
  //unblocked, with replacement
  if(isReplace){
   for(int sample=0; sample<nSamples; sample++){
    int randNum = r.nextInt(nTrials);
    set[counter] = trials[phase][randNum];
    counter++;
    if(isPaired){
     set[counter] = trialsB[phase][randNum];
     counter++;
    }
   }
  }
  //blocked, without replacement
  else{
   for(int block=0; block<nBlocks[phase]; block++){
    for(int trial=0; trial<nTrials; trial++){
     int randNum = r.nextInt(nTrials-trial);
     set[counter] = trials[phase][randNum];
     counter++;
     String temp = trials[phase][randNum];
     trials[phase][randNum] = trials[phase][nTrials-trial-1];
     trials[phase][nTrials-trial-1] = temp;
     //make adjustment for two alternating tasks
     if(Main.twoTasks){
      randNum = r.nextInt(nTrials-trial);
      set[counter] = trialsB[phase][randNum];
      counter++;
      temp = trialsB[phase][randNum];
      trialsB[phase][randNum] = trialsB[phase][nTrials-trial-1];
      trialsB[phase][nTrials-trial-1] = temp;
     }
    }
   }
  }
 }

 /**This method retrieves the indexed trial.
  * @param index The trial number.
  * @return The trail string.*/
 public String get(int index){
  return set[index];
 }

 /**This method splits trials up into two alternating tasks, using a second
  * trial list.*/
 private void splitTrials(){
  //each list will have half the trials
  nTrials = ((nTrials+1) / 2);
  //set second half of trials
  for(int i=0; i<nTrials; i++){
   trialsB[phase][i] = trials[phase][nTrials+i];
  }
 }
}