package pep;

import java.util.Random;

/**This class defines the operations of Input nodes.*/
public class Input extends Node{
 /**stimulus bias*/           private double bias;
 /**stimulus signal*/         private double signal;
 /**random number generator*/ private Random r = new Random();
 
 /**Creates a new Input node.
  * @param subject Subject number used for static variable array.*/
 public Input(int subject){
  super(subject);
  threshold = .5;
  bias = 0;
  signal = 0;
 }

 /**Defines the input values for the node.
  * @param bs Amount of bias for the stimulus.
  * @param sig Amount of signal for the stimulus.*/
 public void turnOn(double bs,double sig){
  setBias(bs);
  signal = sig;
 }

 /**Computes a normally distributed bias value.
  * @param bs Bias multiplier.*/
 private void setBias(double bs){
  //compute normally distributed noise
  bias = range((r.nextGaussian()*.2)+.5)*.8;
  //multiply result by bias multiplier
  bias *= bs;
 }

 /**Calls the superclass method and activates Identity nodes.
  * @param stimNum The stimulus number.
  * @param isConnected Node connected to Identity nodes?*/
 public void activate(int stimNum,boolean isConnected){
  //set input to node and update activation
  setInput();
  update();
  //targets and conflict distracters activate Identity nodes
  if(isConnected) actOut(stimNum);
 }

 /**Calculates and returns output to Episode nodes.
  * @return Output to episodes.*/
 public double getOut(){
  //calculate output
  double output = range(activation-threshold)*strength;
  //return to RunTime
  return output;
 }

 /**Updates bias and sets the input for cycle.*/
 private void setInput(){
  //adjust bias toward signal
  bias = (bias*.996) + (signal*.004);
  //set input with bias and some noise
  input += bias;
  input += range((r.nextGaussian()*.2)+.5)*.5;
 }

 /**Reduces signal and bias when stimulus removed from display.*/
 public void remove(){
  signal *= .98;
  bias *= .98;
 }

 /**Reduces signal and bias when stimulus is masked.*/
 public void mask(){
  signal = 0;
 }

 /**Shuts off signal and bias.*/
 public void shutOff(){
  signal = 0;
  bias = 0;
 }

 /**Sends activation to Identity node.
  * @param stimNum The stimulus number.*/
 private void actOut(int stimNum){
  if(activation>threshold){
   //calculate output
   double output = (activation-threshold);
   //increase output for targets
   if(isTarget) output*=3.2; else output*=1.8;
   output = range(output);
   //activate Identity nodes
   Main.runs[subj].identities[stimNum].input += output;
  }
 }
}