/**Parallel Episodic Processing (PEP) Model
 * @version 2.0.0
 * @author James R. Schmidt <james.schmidt@ugent.be>
 * Universiteit Gent
 * Fonds Wetenschappelijk Onderzoek - Vlaanderen (FWO)*/

package pep;

import java.util.concurrent.Semaphore;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class Main{
 //----------VARIABLES------------------------------------------
 /**DataSet source*/
 static String source = "aPCe2ici.txt";
 /**number of subjects*/              static final int nSubjects = 500;
 /**number of concurrent threads*/    public static final int maxThreads = 3;
 //-------------------------------------------------------------
 /**random with replacement?*/        static boolean isReplace = false;
 /**number of blocks per subject*/    static int[] nBlocks = {5,2,2};
 /**number of no replacement trials*/ static int nSamples = 0;
 /**two alternating tasks?*/          static boolean twoTasks = false;
 /**paired trial order?*/             static boolean isPaired = false;
 //-------------------------------------------------------------
 /**number of responses*/             static int nResponses = 4;
 /**number of targets*/               static int nTargets = 4;
 /**number of distracters per set*/   static final int[] nDistracters = {4};
 /**number conflict stimuli*/         static final int nConflicting = 4;
 /**Set 1 distracter conflict*/       static boolean isConflict = true;
 /**conditions per variable*/         static final int[] numPerVar = {2,2,3,2};
 //-------------------------------------------------------------
 /**target bias*/                     static final double tBias = 1;
 /**target signal*/                   static final double tSignal = 1;
 /**distracter bias*/                 static final double[] dBias = {1};
 /**distracter signal*/               static final double[] dSignal = {1};
 /**degraded signal*/                 static final double tDegrade = 1;
 /**degraded stimuli?*/               static final boolean isDegrade = false;
 //-------------------------------------------------------------
 /**target onset time*/               public static int tOnset = 1;
 /**target offset time*/              public static int tOffset = 2000;
 /**distracter onset time*/           public static int dOnset = 1;
 /**distracter offset time*/          public static int dOffset = 2000;
 /**distracter mask*/                 public static int dMask = 2000;
 /**interstimulus interval*/          public static int isi = 300;
 //-------------------------------------------------------------

 /**thread manager*/
 public static final Semaphore freeThread = new Semaphore(maxThreads);
 /**array of runs used for separate subject threads*/
 public static RunTime runs[] = new RunTime[nSubjects];

 /**For each participant, the main method calls RunTime (the guts of the
  * model), and then generates condition means with DataAnalysis and
  * creates a concatenated datafile with Concat.*/
 public static void main(String[] args) throws InterruptedException{
  //loop through particiapnts
  for(int y=0; y<nSubjects; y++){
   //try to acquire thread, blocking until acquired
   freeThread.acquire();
   //create and run a RunTime thread for a participant
   runs[y] = new RunTime(y);
   Thread t = new Thread(runs[y]);
   //start thread
   t.start();
   System.out.println(""+(y+1)+" of "+nSubjects+" subjects started...");
  }

  //block until all threads are available (i.e., no active threads)
  freeThread.acquire(maxThreads);
  System.out.println("Run complete! Processing data...");

  //analyse data
  DataAnalysis.main(new String[0]);
  System.out.println("Data analysis complete...");

  //create concatenated raw dataset
  Concat.main(new String[0]);
  System.out.println("Data files concatenated...");
 }
}