package pep;

/**This superclass defines the basic fields and methods of Nodes*/
public class Node{
 /**rate of decay*/                   public double decay;
 /**incoming activation*/             public double input;
 /**current activation*/              public double activation;
 /**threshold to pass on activation*/ public double threshold;
 /**search strength*/                 public double strength;
 /**target truth value*/              public boolean isTarget;
 /**subject number*/                  public int subj;

 /**Creates a new Node.
  * @param subject Subject number used for static variable array.*/
 public Node(int subject){
  subj=subject;
  decay = .01;
  input = 0;
  activation = 0;
  isTarget = false;
 }

/**Uses input to update the activation of the Node.*/
 public void update(){
  //apply logistic function
  input = (1 / (1 + Math.exp(-((input*8)-4))));
  //compute activation
  activation = activation*(1-decay) + input*decay;
  //reset input
  input = 0;
 }

 /**Ranges activation between 0 and 1.
  * @param in Value to be ranged.
  * @return The newly ranged value.*/
 public double range(double in){
  if(in<0) return 0;
  else if(in>1) return 1;
  else return in;
 }

 /**Ranges activation between 0 and max.
  * @param in Value to be ranged.
  * @param max Maximum value to return.
  * @return The newly ranged value.*/
 public double range(double in,double max){
  if(in<0) return 0;
  else if(in>max) return max;
  else return in;
 }

 /**Sets memory search strength for target.*/
 public void targeted(){
  //set memory connection strength
  strength = 2;
  //truth value used for several methods
  isTarget = true;
 }

 /**Sets memory search strength for distracter.
  * @param nSets The number of distracter sets.*/
 public void ignored(int nSets){
  //memory connection strength divided accross distracting dimensions
  strength = 5/(double)nSets;
  //truth value used for several methods
  isTarget = false;
 }
}